#ifndef LIBTORRENT_DOWNLOAD_TYPES_H
#define LIBTORRENT_DOWNLOAD_TYPES_H

namespace torrent {

enum heuristics_enum {
  HEURISTICS_UPLOAD_LEECH,
  HEURISTICS_UPLOAD_SEED,
  HEURISTICS_UPLOAD_LEECH_EXPERIMENTAL,
  HEURISTICS_DOWNLOAD_LEECH,
  HEURISTICS_MAX_SIZE
};

}

#endif
